******************************************************
*  Program...........: csZipWrap.prg
*  Unit Test.........: csZipWrap_ut.prg
*  Author............: Lauren Clarke - lauren@cornerstonenw.com
*  Updates...........: www.cornerstonenorthwest.com
*  Created...........: 05/23/2001
*  Description.......: Wraps the Dynazip ActiveX controls
*    csBaseZip.......: Base wrapper class
*    csZip...........: Zip Class
*    csUnZip.........: UnZip Class
******************************************************
* VERSION: 1.0
******************************************************
* Change list
******************************************************

*==============================================
*==============================================
DEFINE CLASS csBaseZip as RELATION
*==============================================
*==============================================
* A place for things common to zip and unzip 

oDZ  		   = NULL
cPassword 	   = ''
lDiag 		   = .f.
cLongMessage   = ''
cMessage 	   = ''
cWrapperMessage = ''
cTempDir 	   = ''
DIMENSION aItems[1]

*****************************************
FUNCTION THIS_ACCESS( tcMember)
*****************************************
IF PEMSTATUS(THIS, UPPER( tcMember),5)
  RETURN THIS
ELSE
  RETURN This.oDZ
ENDIF

*****************************************
FUNCTION SetState( tcStateID)
*****************************************
LOCAL lcStateID
lcStateID = UPPER(ALLTRIM( tcStateID))
This.cTempDir = ADDBS(SYS(2023))

WITH THIS.oDZ
  IF  "DEFAULT" $ lcStateID
      STORE .t. to .BackgroundProcessFlag, ;
      			   .QuietFlag, ;
			       .MessageCallbackFlag, ;
			       .AllQuiet, ;
			       .NoDirectoryNamesFlag   && not sure if this is a good idea
  ENDIF

 IF  "RECURSE" $ lcStateID
      STORE .f. to .NoDirectoryNamesFlag  
      STORE .t. to .RecurseFlag
 ENDIF

  *-- expand here...
ENDWITH
RETURN


*****************************************
FUNCTION AddDBFItem( tcItem)
*****************************************
* adds a dbf and dependencies to the item list

LOCAL lcExt 

lcPath = JUSTPATH(tcItem)
lcStem = JUSTSTEM(tcItem)

* Note: lcStem could = "*"

*--get all dependencies
This.AddItem( lcPath+[\]+lcStem+".dbf")
This.AddItem( lcPath+[\]+lcStem+".fpt")
This.AddItem( lcPath+[\]+lcStem+".cdx")

*****************************************
FUNCTION AddItem( tcItem)
*****************************************
LOCAL lnRow

IF VARTYPE(This.aItems) = "L"
  lnRow = 1
ELSE
  lnRow = ALEN(This.aItems,1)+1
  DIMENSION This.aItems[lnRow]
ENdIF
This.aItems[lnRow]= tcItem
RETURN

*****************************************
FUNCTION GetItemString()
*****************************************
LOCAL lnK, lcRetVal

IF VARTYPE(This.aItems) = "L"
  RETURN ''
ENDIF

lcRetVal = ''
FOR lnK = 1 to ALEN(This.aItems,1)
  lcRetVal = lcRetVal + ["]+This.aItems[lnK]+[" ]
ENDFOR
RETURN lcRetVal

*****************************************
FUNCTION lDiag_ASSIGN( tlValue)
*****************************************
This.lDiag = tlValue
This.oDZ.DiagnosticFlag = tlValue

*****************************************
FUNCTION DESTROY()
*****************************************
This.oDZ = NULL

ENDDEFINE





*==============================================
*==============================================
DEFINE CLASS csZip as csBaseZip
*==============================================
*==============================================
cComment   = ''
cAction = 'ADD'

*****************************************
FUNCTION cMessage_ACCESS
*****************************************
RETURN IIF(!EMPTY(This.cWrapperMessage),This.cWrapperMessage+CHR(13),'')+DZCodeToMessage( This.oDZ.ErrorCode)

*****************************************
FUNCTION cLongMessage_ACCESS
*****************************************
RETURN IIF(!EMPTY(This.cWrapperMessage),This.cWrapperMessage+CHR(13),'')+DZCodeToMessage( This.oDZ.ErrorCode,.t.)


*****************************************
FUNCTION cPassword_Assign( tcValue)
*****************************************
IF !EMPTY( tcValue)
  This.oDZ.EncryptFlag = .t.
ELSE
  This.oDZ.EncryptFlag = .f.
ENDIF
This.oDZ.EncryptCode = tcValue
This.cPassword = tcValue
ENDFUNC

*****************************************
FUNCTION cComment_ASSIGN( tcValue)
*****************************************
IF !EMPTY( tcValue)
  This.oDZ.AddCommentFlag = .t.
ELSE
  This.oDZ.E = .f.
ENDIF
This.oDZ.Comment = tcValue
This.cPassword = tcValue
ENDFUNC

*****************************************
FUNCTION INIT()
*****************************************
*--- instantiate the AX object (zip version)
THIS.oDZ = CREATE( 'dzactxctrl.dzactxctrl.1')
*--- set common property settings
THIS.SetState("DEFAULT")


*****************************************
FUNCTION SetState( tcStateID)
*****************************************
LOCAL lcStateID
lcStateID = UPPER(ALLTRIM( tcStateID))
DODEFAULT( lcStateID)

WITH THIS.oDZ
  IF  "DEFAULT" $ lcStateID

      STORE 1 to .CompressionFactor 
      STORE This.cTempDir to .TempPath
  ENDIF

  *-- expand here...
ENDWITH
RETURN


*****************************************
FUNCTION ZipToFile( tcFile)
*****************************************
* does the deed
LOCAL lcMode, llRetVal, lcOldFile, llWrapperError

lcOldFile = This.ZipFile

lcMode = ALLTRIM(UPPER(This.cAction))

IF VARTYPE( tcFile) = "C"
  This.ZipFile = tcFile
ENDIF

This.oDZ.ItemList = This.oDZ.ItemList + [ ] + This.GetItemString()
DO CASE
CASE lcMode = "FRESHEN"
  This.oDz.ActionDZ=1
*CASE lcMode = "DELETE"		&& we'll make a different method for this
*  This.oDz.ActionDZ=2
CASE lcMode = "UPDATE"
  This.oDz.ActionDZ=3
CASE lcMode = "ADD"
  This.oDz.ActionDZ=4
*-- could expand here with callbacks
OTHERWISE
  THIS.cWrapperMessage=[WRAPPER ERROR: ]+lcMODE+[ is not a recognized action.] 
  llWrapperError = .t.
ENDCASE

This.ZipFile     = lcOldFile

IF llWrapperError
	RETURN .f.
ENDIF
RETURN This.oDZ.ErrorCode = 0

*****************************************
FUNCTION ZIPStrToStr( tcString)
*****************************************
* tcString is passed by reference
lcTempFile = This.cTempDir+SYS(2015)+ALLTRIM(STR(Application.ProcessID))+".zip"

*-- with callbacks, we may be able to do this directly and avoid the FILETOSTR
IF This.ZIPStrToFile( tcString, lcTempFile)
  IF FILE( lcTempFile) 
  	tcString = FILETOSTR( lcTempFile)
  	RETURN .t.
  ELSE
    RETURN .f.
  ENDIF
ELSE
  RETURN .f.
ENDIF

*****************************************
FUNCTION ZIPStrToFile( tcString, tcFileName, tcItemName)
*****************************************
* Creates a ziped file of tcString
tcString = tcString+CHR(0)

loDz = This.oDz
WITH loDZ
   .MemoryBlockSize = LEN(tcString)
IF VARTYPE(  tcItemName)="C"
  .ItemList = tcItemName
ELSE 
  .ItemList = "Str.txt"
ENDIF
IF VARTYPE( tcFileName)="C"
  .ZIPFile  = tcFileName
ENDIF
  .ZipString = tcString
  .ActionDZ=5
ENDWITH
RETURN This.oDZ.ErrorCode = 0

ENDDEFINE


*==============================================
*==============================================
DEFINE CLASS csUnZip as csBaseZip
*==============================================
*==============================================
cAction = 'EXTRACT'

*****************************************
FUNCTION cComment_ACCESS
*****************************************
RETURN This.oDZ.ReturnString

*****************************************
FUNCTION cMessage_ACCESS
*****************************************
RETURN IIF(!EMPTY(This.cWrapperMessage),This.cWrapperMessage+CHR(13),'')+DUZCodeToMessage( This.oDZ.ErrorCode)

*****************************************
FUNCTION cLongMessage_ACCESS
*****************************************
RETURN IIF(!EMPTY(This.cWrapperMessage),This.cWrapperMessage+CHR(13),'')+DUZCodeToMessage( This.oDZ.ErrorCode,.t.)

*****************************************
FUNCTION cPassword_Assign( tcValue)
*****************************************
IF !EMPTY( tcValue)
  This.oDZ.DecryptFlag = .t.
ELSE
  This.oDZ.DecryptFlag = .f.
ENDIF
This.oDZ.DecryptCode = tcValue
This.cPassword = tcValue
ENDFUNC

*****************************************
FUNCTION INIT()
*****************************************
*--- instantiate the AX object (zip version)
THIS.oDZ = CREATE( 'duzactxctrl.duzactxctrl.1')
*--- set common property settings
THIS.SetState("DEFAULT")

*****************************************
FUNCTION UnZIPFile( tcFile, tcDestination)
*****************************************
LOCAL lcMode, lcOldFile, lcOldDest, llWrapperError

lcOldFile = This.ZipFile
lcOldDest = This.Destination

lcMode = ALLTRIM(UPPER(This.cAction))

IF VARTYPE( tcFile) = "C"
  This.ZipFile = tcFile
ENDIF

IF VARTYPE( tcDestination) = "C"
  This.Destination = tcDestination
ENDIF

This.oDZ.FileSpec = This.oDZ.FileSpec + [ ]+ This.GetItemString()

IF EMPTY(This.oDZ.FileSpec)
  This.oDZ.FileSpec = "*.*"
ENDIF

DO CASE
CASE lcMode = "EXTRACT"
  This.oDz.ActionDZ=8
OTHERWISE
  THIS.cWrapperMessage=[WRAPPER ERROR: ]+lcMODE+[ is not a recognized action.] 
  llWrapperError = .t.
ENDCASE

This.ZipFile     = lcOldFile
This.Destination = lcOldDest

IF llWrapperError
	RETURN .f.
ENDIF
RETURN This.oDZ.ErrorCode = 0


*****************************************
FUNCTION UnZIPStrToStr( tcString)
*****************************************
* tcString passed by reference
lcTempFile = This.cTempDir+SYS(2015)+ALLTRIM(STR(Application.ProcessID))+".zip"
*-- with callbacks, we may be able to do this directly and avoid the STRTOFILE
STRTOFILE( tcString, lcTempFile)
This.UnZIPFileToStr( @tcString, lcTempFile)
RETURN

*****************************************
FUNCTION UnZIPFileToStr( tcString, tcFileName, tcItemName)
*****************************************
* note: tcString should be passed by reference

LOCAL lnOffest, lnBytestToRead, lcCurrentRead

IF VARTYPE(  tcItemName)="C"
  This.oDZ.FileSpec = tcItemName
ELSE
  This.oDZ.FileSpec = "Str.txt" 
ENDIF
IF VARTYPE( tcFileName)="C"
  This.oDZ.ZIPFile  = tcFileName
ENDIF

tcString = ''

*--- grab a local pointer for efficiency
loDZ = This.oDz
WITH loDZ
  .UnZipIndex = 0		&& point at the first item
  .ActionDZ = 7		&& get info
  .UnZipStringSize = .zi_oSize

  .ActionDZ = 9
  tcString = .UnZipString
ENDWITH 
RETURN loDZ.ErrorCode = 0

*!*	WITH loDZ
*!*	  .UnZipStringSize = lnBytesToRead
*!*	  DO WHILE .UnZipStringSize=lnBytesToRead
*!*	    .UnZipStringOffset = lnOffset
*!*	    .ActionDZ=9	&& extracts the next lnBytesToRead
*!*	    lcCurrentRead = .UnZipString
*!*	    tcString = tcString + lcCurrentRead
*!*	    lnOffset = lnOffset + lnBytesToRead+1
*!*	  ENDDO 
*!*	ENDWITH 

ENDDEFINE


*******************************************************************
* UDF's follow
*******************************************************************

FUNCTION DUZCodeToMessage( tnCode, tlVerbose)
LOCAL lcRetVal

DO CASE
CASE tnCode = 0
   lcRetVal = [Successful operation -  From DynaZIP's point of view everything went OK.]
CASE tnCode = 2
   lcRetVal = [Unexpected end of ZIP file - The ZIP file may be corrupted, you tried to extract more data than was found for the specified item in the ZIP file.	]
CASE tnCode = 3 
   lcRetVal = [ZIP file structure invalid - The internal organization of the ZIP file has been damaged.  Try fixing the ZIP file.  Be sure to make a backup copy first.]
CASE tnCode =   4
   lcRetval = [Out of memory - Too many applications running.  Close some applications and try again.]
CASE tnCode =   5 
   lcRetval = [Out of memory - Too many applications running. Close some applications and try again.]	
CASE tnCode =   9 
   lcRetval = [File not found - You may have forgotten to provide a full path with drive letter, path, etc.  ]+;
              [Or the ZIP file could not be opened because it is locked by another application or the directory permissions are too restrictive.]
CASE tnCode = 11  
   lcRetval = [No files to test, extract, etc. - The filespec does not match precisely any items that are in the ZIP file.  ]+ ;
              [This may be because you forgot to set the RecurseFlag property to True or you made an error in specifying the list of items in the Filespec property.  ]+;
              [For example, you may have forgotten to use double quote characters around an item name that contains a space character.]
CASE tnCode = 12  
   lcRetval = [This error occurs when you specify a Destination that is the same as the drive on which the ZIP file is located.  ]+;
   			  [Since you may be removing the media, you can't extract to the same volume.  This may also be caused when you set the ]+;
   			  [current directory to the same drive as the one containing the ZIP file and you try to extract to a UNC based path which has no corresponding drive letter.]

CASE tnCode = 25  
   lcRetval = [Index out of bounds - The index value you specified does not fall within the range of expected values. Remember, the range of index values can change based on the UnZipSubOptions setting for USO_IGNORELONGNAMES.]CASE tnCode = 14  
CASE tnCode = 28
   lcRetval = [Error creating output file - During the extraction process DynaZIP must be able to exclusively access and create a file ]+;
              [which has a name formed by the Destination and internal zip item name.  If a destination directory is locked, or a file by the same ]+;
              [name is locked or marked as read only, then the creation of the output file is impossible.  You can try setting the USO_OVERWRITE_RO option in the UnZipSubOptions Property.	]
CASE tnCode = 29  
   lcRetval = [Error opening archive - The ZIP file that you specified could be found but is locked or unavailable.  Perhaps the SECURITY permissions are restricting access to the ZIP file, or the directory that contains the file is locked or restricted.	]
CASE tnCode = 39 
   lcRetval = [Bad CRC - When a file is zipped, an internal CRC calculation is made for the file's data.  When the item is extracted, the same CRC calculation is performed.  If the data has changed or become corrupted then the original CRC value will not ]+;
              [match the currently calculated CRC value.  This indicates a data problem and the extracted file should be suspect.]
CASE tnCode = 40 
   lcRetval = [Application canceled operation - The user has canceled the extraction operation or there is a coding error in the MajorStatus, ]+;
              [MinorStatus or MessageCallback Event Procedures.	]

CASE tnCode = 41  
   lcRetval = [File skipped, encrypted - The file you are attempting to extract is encrypted.  Either the password is incorrect or was not provided.  Recheck the password.	]
CASE tnCode = 42  
   lcRetval = [File skipped, unknown compression method - The ZIP file is incompatible with the PKWARE 2.04g standard.]	

CASE tnCode = 44  
   lcRetval = [Bad or missing decrypt code - The file you are attempting to extract is encrypted.  Either the password is incorrect or was not provided.  Recheck the password.]
CASE tnCode = 45  
   lcRetval = [Busy, can't enter now - You tried to initiate another unzip action within the same thread of execution before the last operation completes.  This could happen if your program tries to unzip from within one of the event handlers.]
CASE tnCode = 46  
   lcRetval = [Can't extract a Volume ID item - The ZIP file contains a Volume label, however when an extraction of the Volume label occurred it could not written to the media.]	
CASE tnCode = 47  
   lcRetval = [Bad command structure - You have specified an inappropriate combination of properties or have omitted a property setting that is required for a particular operation.]	
CASE tnCode = 48  
   lcRetval = [Operation canceled - User canceled through one of DynaZIP's internal operations.]	
CASE tnCode = 49  
   lcRetval = [Operation skipped - One or more of the items that was specified for extration was skipped.  Usually this occurs when you set the QuietFlag and the password does not match the required password for one or more of the items in the ZIP file.]	
CASE tnCode = 50  
   lcRetval = [Disk full - The extraction process has filled the destination volume to capacity. You will need to remove some files on the target volume and unzip again.]	   

OTHERWISE
   lcRetval =TRANSFORM(tnCode) + [: unrecognized error code]
ENDCASE

IF tlVerbose
  RETURN lcRetval
ELSE
  RETURN LEFT(lcRetval,AT(" -",lcRetval))
ENDIF
ENDFUNC


FUNCTION DZCodeToMessage( tnCode, tlVerbose)
LOCAL lcRetVal

DO CASE
CASE tnCode = 0
   lcRetVal = [Successful operation -  From DynaZIP's point of view everything went OK.]
CASE tnCode = 1 
   lcRetVal = [Busy, can't enter now - You tried to initiate another zip action within the same thread of execution before the last operation completes.  This could happen if your program tries to zip from within one of the event handlers.]
CASE tnCode = 2
   lcRetVal = [Unexpected end of ZIP file - When you add to an existing ZIP file, the original ZIP file contents are tested and intersecting item entries are removed so they can be replaced with new data. If the existing Zip file is corrupted, ]+ ;
              [then the process of copying existing entries could fail as DynaZIP tries to extract more data than was found for the specified item in the ZIP file.]
CASE tnCode = 3 
   lcRetVal = [ZIP file structure invalid - The internal organization of the existing ZIP file has been damaged. Try fixing the ZIP file.  Be sure to make a backup copy first.]
CASE tnCode =   4
   lcRetval = [Out of memory - Too many applications running.  Close some applications and try again.]
CASE tnCode =   5 
   lcRetval = [Internal logic error - Contact Inner Media Technical Support immediately if this error occurs.]	
CASE tnCode =   6 
   lcRetval = [Entry too big to split - This error will never occur.  It is documented for backward compatibility only.]
CASE tnCode =   7 
   lcRetval = [Invalid comment format - The comment size is too large or not properly formatted for use in a ZIP file.  Comments must be less than 64K in length.]
CASE tnCode =   8 
   lcRetval = [ZIP file invalid or insufficient memory - While testing the ZIP file, an internal error occurred.  Try fixing the ZIP file.  Be sure to make a backup copy first.]
CASE tnCode =   9 
   lcRetval = [Application canceled operation - The user has canceled the zip operation.  This can also be caused by a coding error in the MajorStatus, MinorStatus or MessageCallback events.  If your event handlers return cancel then you may abort prematurely.]

CASE tnCode = 10  
   lcRetval = [Temporary file failure - When DynaZIP attempted to create a temporary copy of an existing ZIP file an error occurred.  ]+ ;
              [This error could be caused by specifying a non existant path or drive letter in the ZIPFile property.  It can also be caused if the ZIPFile ] + ;
              [is target to a locked or restricted directory or to a file by the same name that is locked or restricted.  Check ] + ;
              [ReadOnly attributes and the possibility of other applications accessing the ZIP file or the directory that contains the ZIP file.]
CASE tnCode = 11  
   lcRetval = [Input file read failure - This error occurs when a file read operation is performed on a ZIP file that is locked or restricted by another process.]
CASE tnCode = 12  
   lcRetval = [Nothing to do - The ItemList that you specified doesn't match any files to be zipped.  This error may be encountered if you are ]+ ;
              [freshening or updating items in the ZIP file and none of the items are actually refreshed.  This error may also occur if you specify the ZSO_JUST_ARCHIVED setting and none of the files to be zipped have changed. ]+; 
              [You may also get this error if you specify a date range for zipping and none of the files to be zipped are within that range.]

CASE tnCode = 13  
   lcRetval = [Missing or empty ZIP file - You have specified a ZIPFile name that does not exist.  Be sure to use full paths with drive letters or UNC paths when specifying the ZIPFile name.]
CASE tnCode = 14  
   lcRetval = [Output file write failure, possible disk full - This error can occur when a file write operation is performed on a ZIP file that is locked or restricted by another process.]
CASE tnCode = 15  
   lcRetval = [Could not create output file - This error can occur when the target ZIP file or the directory to contain the ZIP file is locked or restricted by another.]
CASE tnCode = 16  
   lcRetval = [Invalid combination of control parameters - You have specified an inappropriate combination of properties or have omitted a property setting that is required for a particular operation.]
CASE tnCode = 17  
   lcRetval = [Could not complete operation - At a critical point within a zip operation, the process could not complete.  You should retry the same zip operation and if the problem persists contact Inner Media Technical Support immediately.]

CASE tnCode = 18  
   lcRetval = [File not found or no read permission - The ZIP file that you specified could be found but is locked or unavailable.  ]+;
              [Perhaps the SECURITY permissions are restricting access to the ZIP file or the directory that contains the file is locked or restricted.  It is also possible that the file specification is confused with another ]+;
              [open or restricted file because the file spec contains a space and is not surrounded with double quote characters.]
CASE tnCode = 19  
   lcRetval = [Media error, disk not ready, or hardware error - During a multi-volume operation, the media was found to be corrupted beyond repair or perhaps the user simply opened the drive door during a format or wipe operation.  Try replacing the media.]
CASE tnCode = 20  
   lcRetval = [Invalid combination of multi-volume control parameters - Some of the Multi-Volume options are exclusive and can only be specified in certain combinations.  For example you can not Format and Wipe at the same time.  ]+ ;
		 	  [Please refer to the DynaZIP manual for the restrictions that apply to the MultiVolumeControl property settings.]

CASE tnCode = 21  
   lcRetval = [Improper use of a multi-volume ZIP file - You tried to use a Multi-Volume ZIP file like a non-multi-volume ZIP file. For example, you can not grow an existing multi-volume ZIP file.]
OTHERWISE
   lcRetval =TRANSFORM(tnCode) + [: unrecognized error code]
ENDCASE

IF tlVerbose
  RETURN lcRetval
ELSE
  RETURN LEFT(lcRetval,AT(" -",lcRetval))
ENDIF
ENDFUNC

