* unit test for csZipWrap
* Note: You'll probably want to change some of the paths below

#DEFINE CODE_DIR [d:\csnwclasses\]
#DEFINE TEMP_DIR [d:\temp\]		
#DEFINE VERBOSE .t.
#DEFINE HALT_ON_ERROR .f.
*------------------------------------------

*------------------------------------------
* SETUP REGION
*------------------------------------------
SET PROC TO CODE_DIR+[zipwrap.prg]

*------------------------------------------
* PREPARE FOR TESTS
*------------------------------------------
LOCAL loZ, loUZ, lcTestID
CLEAR

* set up some sample files
LOCAL lcStr
close tables

lcCurrentTempDir = TEMP_DIR+SYS(2015)
MkDIR &lcCurrentTempDir
lcStr = ''
FOR lnk = 1 to 1000
  lcStr = lcStr+SYS(2015)
ENDFOR
STRTOFILE(lcStr,lcCurrentTempDir+"\Text1.txt")
STRTOFILE(REPLICATE(lcStr,10),lcCurrentTempDir+"\Text2.txt")
MKDIR lcCurrentTempDir+"\DIR1"
STRTOFILE(REPLICATE(lcStr,5),lcCurrentTempDir+"\DIR1\text2.txt")
lcTableName = lcCurrentTempDir+"\zipTest.dbf" 
CREATE TABLE &lcTableName (ID CHR(10), cText M)
INDEX on ID TAG ID
FOR lnK = 1 to 100
  INSERT INTO zipTest (ID,cText) VALUES (SYS(2015),REPLICATE(SYS(2015),100))
ENDFOR


*------------------------------------------
* csZip AddItem/GetItemString
*------------------------------------------
* Checks the AddItem and GetItemString methods


lcTestID 		= 	[csZip AddItem/GetItemString: Single Item]
loZ				= 	CREATEOBJECT("csZip")

loZ.AddItem("d:\FirstDir\FirstFile.txt")
=Test(lcTestID,line(),loZ.GetItemString(),["d:\FirstDir\FirstFile.txt" ])

lcTestID 		= 	[csZip AddItem/GetItemString: Multiple Items]
loZ				= 	CREATEOBJECT("csZip")
loZ.AddItem("d:\FirstDir\FirstFile.txt")
loZ.AddItem("d:\SecondDir\SecondFile.txt")
=Test(lcTestID,line(),loZ.GetItemString(),["d:\FirstDir\FirstFile.txt" "d:\SecondDir\SecondFile.txt" ])

lcTestID 		= 	[csZip AddItem/GetItemString: Items with spaces]
loZ				= 	CREATEOBJECT("csZip")
loZ.AddItem("d:\First Dir\First File.txt")
loZ.AddItem("d:\Second Dir\Second File.txt")
=Test(lcTestID,line(),loZ.GetItemString(),["d:\First Dir\First File.txt" "d:\Second Dir\Second File.txt" ])



*------------------------------------------
* csZip AddDbf/GetItemString
*------------------------------------------
* Checks the AddItem and GetItemString methods


lcTestID 		= 	[csZip AddDbf/GetItemString: Single Item]
loZ				= 	CREATEOBJECT("csZip")

loZ.AddDBFItem("d:\SomeDir\customers.dbf")
=Test(lcTestID,line(),loZ.GetItemString(),["d:\SomeDir\customers.dbf" "d:\SomeDir\customers.fpt" "d:\SomeDir\customers.cdx" ])

*------------------------------------------
* csZip ZipToFile
*------------------------------------------
*
lcTestID 		= 	[csZip ZipToFile]
loZ				= 	CREATEOBJECT("csZip")
loZ.Additem(lcCurrentTempDir+"\Text1.txt")
llResult = loZ.ZipToFile(lcCurrentTempDir+"\Text1.zip")
=Test(lcTestID,line(),llResult,.t.)

*------------------------------------------
* csUNZip ZipToFile
*------------------------------------------
*
lcTestID 		= 	[csZip UnZipToFile]
loZ				= 	CREATEOBJECT("csUnZip")
llResult = loZ.UnZipFile(lcCurrentTempDir+"\text1.zip", lcCurrentTempDir+"\text1\")
=Test(lcTestID,line(),llResult,.t.)
lcTestID 		= 	[csZip UnZipToFile: Check File exists]
llResult = FILE(lcCurrentTempDir+"\Text1\Text1.txt")
=Test(lcTestID,line(),llResult,.t.)
ERASE lcCurrentTempDir+"\Text1\Text1.txt"
RMDIR lcCurrentTempDir+"\Text1"


*------------------------------------------
* csZip ZipToFile with password
*------------------------------------------
*
lcTestID 		= 	[csZip ZipToFile with password]
loZ				= 	CREATEOBJECT("csZip")
loZ.cPassword	=   "zipwrap"
loZ.Additem(lcCurrentTempDir+"\Text1.txt")
llResult = loZ.ZipToFile(lcCurrentTempDir+"\Text1.zip")
=Test(lcTestID,line(),llResult,.t.)

*------------------------------------------
* csUNZip ZipToFile with password
*------------------------------------------
*
lcTestID 		= 	[csZip UnZipToFile with password]
loZ				= 	CREATEOBJECT("csUnZip")
loZ.cPassword   =   "zipwrap"
loZ.Allquiet  = .f.
loZ.QuietFlag = .f.
llResult = loZ.UnZipFile(lcCurrentTempDir+"\text1.zip", lcCurrentTempDir+"\text1\")
=Test(lcTestID,line(),llResult,.t.)
lcTestID 		= 	[csZip UnZipToFile with password: Check File exists]
llResult = FILE(lcCurrentTempDir+"\Text1\Text1.txt")
=Test(lcTestID,line(),llResult,.t.)
ERASE lcCurrentTempDir+"\Text1\Text1.txt"
RMDIR lcCurrentTempDir+"\Text1"






*------------------------------------------
* csZip AddDBFItem and then ZIP
*------------------------------------------
close tables
lcTestID 		= 	[csZip AddDBFItem with ZIP]
loZ				= 	CREATEOBJECT("csZip")
loZ.AddDBFITEM(lcCurrentTempDir+"\zipTest")
llResult = loZ.ZipToFile(lcCurrentTempDir+"\zipTest")
=Test(lcTestID,line(),llResult,.t.)
ERASE lcCurrentTempDir+"\zipTest.zip"

*------------------------------------------
* csZip.ZipStrToFile/csunZip.UnZipFileToStr 
*------------------------------------------
* Checks that ZipStrToFile and UnZipFIleToStr are inverses

lcTempDir		=   ADDBS(SYS(2023))
lcTestID 		= 	[csZip ZipStrToFile]
loZ				= 	CREATEOBJECT("csZip")

lcText 			=   "This is a test"
llResult = loZ.ZipStrToFile(lcText,lcTempDir+"test.zip","Test.abc")
=Test(lcTestID,line(),llResult,.t.)

lcTestID 		= [csUnZip UNZipFileToStr]
lcTemp          = ''
loUZ			= CREATEOBJECT("csUnZip")
lnResult = loUz.UnZIPFileToStr( @lcTemp, lcTempDir+"test.zip","Test.abc")
=Test(lcTestID,line(),llResult,.t.)
lcTestID 		= [ZipStrToFile/UNZipFileToStr Compare]
=Test(lcTestID,line(),lcTemp,lcText)

*------------------------------------------
* csZip.ZipStrToStr/csunZip.UnZipStrToStr 
*------------------------------------------
* Checks that ZipStrToStr and UnZipStrToStr are inverses

lcTempDir		=   ADDBS(SYS(2023))
lcTestID 		= 	[csZip ZipStrToStr]
loZ				= 	CREATEOBJECT("csZip")

lcOrigText		=   FILETOSTR(lcCurrentTempDir+"\text2.txt")
lcText          =  lcOrigText

llResult = loZ.ZipStrToStr( @lcText)
=Test(lcTestID,line(),llResult,.t.)

lcTestID 		= [csUnZip UNZipStrToStr]
lcTemp          = ''
loUZ			= CREATEOBJECT("csUnZip")
lnResult = loUz.UnZIPStrToStr( @lctext)
=Test(lcTestID,line(),llResult,.t.)
lcTestID 		= [csUnZip ZipStrToStr/UNZipStrToStr Compare]
=Test(lcTestID,line(),lctext,lcOrigText)

****************************************************************************************************
*-------------------------
* TEST UDF
*-------------------------
Function Test( tcTest, tnLine, tvTestReturn, tvTestTarget)

IF tvTestReturn # tvTestTarget
	Activate Screen
	? [TestFailure: ]+tcTest+[ at ],tnline
	IF HALT_ON_ERROR
		Cancel
	ENDIF
ELSE
	IF VERBOSE
		? [Test Pass: ]+tcTest
	ENDIF
ENDIF
RETURN


